
<%@ Register TagPrefix="Acme" TagName="SourceRef" Src="/quickstart/util/SrcRef.aspx"%>

<!-- #include virtual="/quickstart/aspplus/include/header.inc" -->

<h4>Working With Business Objects</h4>
<p>

<!-- #include virtual="/quickstart/aspplus/include/wftoc5-1.inc" -->
<p>
<hr>

Encapsulating logic in business components is an essential part of any real-world application, web-based or otherwise.  In ASP+, business
objects are the building blocks for multi-tiered web applications, such as those with a layer for data access or common application rules.
In this section we demonstrate how to write some simple components and include them in your application's Web Form pages.
<p>

<!--BEGIN SECTION-->
<a name="bindir">
<span class="subhead">The Application /Bin Directory</span>
<p>
A problem with using the classic COM model for web application components is that those components must be registered (typically 
using the regsvr32 utility) before they can be used from a traditional ASP application.  Remote administration of these types of 
applications is often not possible, because the registration utility must be run locally on the server.  To make matters more difficult,
these components remain locked on disk once they are loaded by an application, and the entire web server must be stopped before
these components can be replaced or removed.
<p>
ASP+ attempts to solve these problems by allowing components to be placed in a well-known directory, to be automatically found
at run-time.  This well-known directory is always named <b>/bin</b>, and is located immediately under the root directory for the 
application (a virtual directory defined by IIS).  The benefit is that no registration is required to make components available to the 
ASP+ application -- components may be deployed by simply copying to the /bin dir or performing an FTP file transfer.
<p>
In addition to providing a zero-registration way to deploy compiled components, ASP+ does not require these components to remain
locked on disk at runtime.  Behind the scenes, ASP+ duplicates the assemblies found in /bin and loads these "shadow-copies" instead.  
The original components may be replaced even while the web server is still running, and changes to the /bin directory are automatically 
picked up by the runtime.  When a change is detected, ASP+ will gracefully allow currently executing requests to complete, and direct
all new incoming requests to the application which uses the new component(s).


<!--BEGIN SECTION-->
<br>
<a name="importing">
<br>
<span class="subhead">Importing Business Objects</span>
<p>
At its most basic level, a business component is just a class which can be instantiated from a Web Form which imports it.  In this 
example, we define a simple HelloWorld class.  Our class has one public constructor (which is executed when the class is first 
instantiated), a single String property called "FirstName", and a "SayHello" method which prints a greeting using the value of the 
FirstName property.
<p>

<div class="code"><pre>
namespace HelloWorld 
{
using System;
using System.Text;

  public class HelloObj 
  {
    private String _name;

    public HelloObj() {
        _name = null;      
    }

    public String FirstName 
    {
      get {  return _name; }
      set {  _name = value; }
    }

    public String SayHello() 
    {
      StringBuilder sb = new StringBuilder("Hello ");
      if (_name != null)
         sb.Append(_name);
      else
         sb.Append("World");
      sb.Append("!");
      return sb.ToString();
    }
  }
}
</pre></div>

<p>
To compile this class, we run the C# compiler (csc.exe) from the command-line.  The <b>/t</b> option tells the compiler we want to 
build a library (dll), and the <b>/out</b> option tells the compiler where to place the resulting assembly.  In this case, the /bin 
directory for our application is directly under the "aspplus" vroot of this tutorial.
<p>

<div class="code"><pre>
csc /t:library /out:..\..\..\bin\HelloObj.dll HelloObj.cs
</pre></div>

<p>
The component is now available to any Web Form in the application that wishes to use it.  The following HelloObj.aspx example illustrates 
this:
<p>

<Acme:SourceRef 
  RunSample="/quickstart/aspplus/samples/webforms/busobjs/HelloObj.aspx" 
  ViewSource="/quickstart/aspplus/samples/webforms/busobjs/HelloObj.src"
  Icon="/quickstart/aspplus/images/helloobj.gif"
  Caption="HelloObj.aspx"
  runat="server" />


<p>
Note the following Import directive at the top of the page that specifies the namespace to include.  Once the namespace is included using this directive,
 the class may be used from within the Web Form.  Because the assembly is pre-loaded by the ASP+ runtime, only
a simple namespace import is required to make the component available.
<p>

<div class="code"><pre>
&lt;%@ Import Namespace="HelloWorld" %&gt;
</pre></div>

<p>
By default, ASP+ loads all assemblies from the /bin directory when the application is started.  The assemblies to load are specifed via 
the configuration system (see the <a href="configoverview.aspx">Configuration Overview</a> section for more detail).  Additional
assemblies may be imported into an application using configuration as well.  For example:
<p>

<div class="code"><pre>
&lt;configuration&gt;
    &lt;compilation&gt;
        &lt;assemblies&gt;
            &lt;!--The following assemblies are loaded explicitly from the global cache--&gt;
            &lt;add assembly="System.Data"/&gt;
            &lt;add assembly="System.Web.Services"/&gt;
            &lt;add assembly="System.Drawing"/&gt;
            &lt;!--This tells ASP+ to load all assemblies from /bin--&gt;
            &lt;add assembly="*"/&gt; 
        &lt;/assemblies&gt;
    &lt;/compilation&gt;
&lt;/configuration&gt;
</pre></div>

<b>Note: </b> Each assembly loaded from /bin is limited in scope to the application in which it is running. This means that peer applications could 
potentially use different assemblies with the same class or namespace names, without conflicting. 

<!--BEGIN SECTION-->
<br>
<a name="twotier">
<br>
<span class="subhead">A Simple Two-Tier Web Form</span>
<p>

The classic use for an external component is to perform data access.  This simplifies the code in your page, improving readability and 
separating your user interface (UI) logic from the backend functionality.  The following example demonstrates a simple 
two-tiered Web Form using a data access component to retrieve product information.
<p>

<Acme:SourceRef 
  RunSample="/quickstart/aspplus/samples/webforms/busobjs/TwoTier.aspx" 
  ViewSource="/quickstart/aspplus/samples/webforms/busobjs/TwoTier.src"
  Icon="/quickstart/aspplus/images/twotier.gif"
  Caption="TwoTier.aspx"
  runat="server" />

<p>
The data access component takes a single parameter to its constructor specifying the connection string to the product database.
The Web Form calls the component's GetCategories method to populate a dropdown list, and calls the component's GetProductsForCategory
method to display the products for the category selected by the user.

<!--BEGIN SECTION-->
<br>
<a name="threetier">
<br>
<span class="subhead">A Simple Three-Tier Web Form</span>
<p>
A three-tiered application model extends the two-tiered scenario to include business rules between the UI and data access logic.  This 
model allows UI developers to work with a higher level of abstraction rather than directly manipulating data through low-level data 
access component APIs.  The middle business component typically enforces business rules and ensures that the relationships and 
primary key restraints of the database are honored.  In our simple example, the middle component calculates a discount based on 
a two-digit Vendor ID entered by the client.

<p>

<Acme:SourceRef 
  RunSample="/quickstart/aspplus/samples/webforms/busobjs/ThreeTier.aspx" 
  ViewSource="/quickstart/aspplus/samples/webforms/busobjs/ThreeTier.src"
  Icon="/quickstart/aspplus/images/threetier.gif"
  Caption="ThreeTier.aspx"
  runat="server" />


<!--BEGIN SECTION-->
<a name="endofsection">

<h4>Section Summary</h4>
<ol>
<li>The ASP+ runtime finds business objects (local assemblies) in a well-known /bin directory, directly under the application root. The /bin directory offers the following advantages: <p>
<ul>
<li><b>No Registration Required.</b> No registration is required to make an assembly available to pages in the application. It is available by virtue of its location in the /bin directory. Compiled code can be deployed by simply copying or FTP'ing to this location. 
<li><b>No Server Restart Required.</b> When any part of an ASP+ application is changed (for example, replacing a .dll in /bin), new requests immediately begin execution against the changed file(s). Currently executing requests are allowed to complete before the old application is gracefully torn down. The web server does not require a restart when you change your application, even when replacing compiled code. 
<li><b>No Namespace Conflicts.</b> Each assembly loaded from /bin is limited in scope to the application in which it is running. This means that peer applications could potentially use different assemblies with the same class or namespace names, without conflicting. 
</ul>
<p>
<li>Classes in an assembly are made available to a page in the application using an Import directive within the aspx file. 
<li>Two-tiered applications simplify the code in a page, improving readability and separating user interface (UI) logic 
from backend functionality.
<li>Three-tiered applications extend the two-tiered model to enable UI developers to work with a higher level of abstraction. The middle 
business component typically enforces business rules and ensures that the relationships and primary key restraints of the database are 
honored.
</ol>
<p>

<!-- #include virtual="/quickstart/aspplus/include/footer.inc" -->